          SECTION J                                               FILE: MORE.J
                                    WRITING GAMES
                                    *************

              One of the occupations favoured by programmers is writing games.
          There is no end to the variety of games that can be written to be
          played on a computer, from the very simple to the extremely complex.
          There is much of interest here for the beginner as well as the
          professional.
              Any grouping of types of game must be arbitrary, but I suggest
          the following breakdown :-

              1) Games of chance ; These are games such as Poker dice,
          Pontoon, tossing coins, Beetle, etc., They can often be enhanced by
          adding a betting element.
              2) Games of skill ; Chess, Bridge, Draughts etc.
              3) Board games ; Snakes and ladders, Ludo, Backgammon etc.;
          skill and luck combined.
              4) Puzzles ; Towers of Hanoi, Anagrams, Crosswords, etc.
              5) Adventure ; Similar to reading an adventure novel, but you
          have an input which affects the ending.
              6) Simulation ; Driving a racing car or train, flying a 'plane,
              7) Arcade games ; Zapping aliens, jumping motorcycles:- all
          those games you might play in an arcade.

              Many, if not most, games will not fall neatly into a single
          category, but will contain elements of two or more. When setting out
          to write a game, the limitations of your computer must be
          considered.
              The PCW has a useful amount of memory, even the PCW 8256, and it
          has a high resolution screen, which is an advantage when graphics
          are needed. On the other hand the PCW is a rather sedate machine and
          this is emphasised when MALLARD BASIC alone is used; even when
          machine code is used it is often not fast enough for some games and
          simulations, and the lack of colour in the screen spoils some games
          for some people.
              MALLARD BASIC as supplied does not support true graphics
          although there are ways of getting a graphics output from within
          MALLARD.
              One way is to use the CP/M graphics system extension, GSX, which
          is on side 3 of the discs supplied with your PCW (file GSX.SYS) and
          can be grafted on to MALLARD BASIC to give a graphics application,
          but frankly it is so difficult to use that I would advise you to
          avoid it.
              Alternatively graphics output can be achieved through machine
          code, which can be called from within MALLARD BASIC and this is
          especially of interest when high resolution graphics and fast
          movement is needed.
              Another way is to buy a proprietary extension to MALLARD BASIC
          itself. Such programs e.g Lightning Basic, are advertised in PCW
          magazines and work by adding new commands for drawing lines, circles
          etc. but as far as I am aware these programs will only meet
          relatively simple requirements.

          WHAT SORT OF GAME IS SUITABLE FOR THE PCW AND MALLARD ?
          =====================================================

              In this file I am only concerned with MALLARD BASIC as supplied,
          and this cuts down the types of game that can be written. We shall
          be looking at games that use mainly text and mathematics, but even
          within this framework the field is still wide. An arcade game
          without graphics is hardly imaginable and the same goes for
          simulation, but most games in the other categories can be
          accommodated with a little forethought and imagination. For example
          cards can be represented by letters e.g QH - Queen of Hearts, 2D -
          two of diamonds, etc. Crude pictures can be attempted where
          absolutely necessary, using letters and symbols to make matchstick
          men and animals.
              Adventure games are very good, because, like a fiction book, no
          pictures are necessary, and indeed many people think that pictures
          detract from the game as they do not allow the player's imagination
          to work.
              A game does not have to be long and complicated to have lasting
          appeal, but it must present a challenge and be a little tantalizing.
          Many of the most popular and oldest games, such as chess, football,
          tennis, bridge, are very simple in concept, but take a long time to
          master. Try to remember that when you write your games; keep the
          rules simple and make sure that the object of the game is explained
          clearly.
              We shall examine a few types of game now and discover how to set
          about writing them.

          GAMES OF CHANCE
          ***************
              A major need of this type of game is the facility to generate
          numbers in random order. For example, many games proceed on the
          throw of a dice, or turning up a card at random. This requirement is
          more difficult than it may seem as so-called 'random' numbers
          generated by a computer are in fact worked out by a mathematical
          formula which always churns out the same series of numbers if
          conditions are identical. However it is fairly easy to achieve a
          randomness which is acceptable for games.
              RND is the main keyword used to generate random numbers. Load
          BASIC then copy out the following short program.
              20 for a=1 to 10
              30 print rnd;
              40 next

              Run this program several times and you will see that the same
          series of numbers is printed each time. If you wish, change the 10
          in line 20 to 100 to get a larger sample, but don't forget the
          semicolon at the end of line 30!.Now change line 30 to:-
              30 print int(rnd*6+1);  and run it several times. This time the
          numbers will be selected from 1 to 6 inclusive and could therefore
          be used to simulate the throw of a dice, except that the series
          again remains the same. Each number is generated from the previous
          number by a mathematical formula; if RND can be started with a
          different number the series will be different. This is called
          'seeding'.
              There are several ways to provide a seed. Start with my
          favourite; add line 10 as follows and change the program to:-
              10 while inkey$="": b=rnd: wend
              20 for a=1 to 10
              30 b=int(rnd*6+1):print b;
              40 next

              When run this program will await a keypress and then print out a
          series of numbers, but each time the program is run the series will
          be different. This is because whilst the keypress was awaited, b=RND
          in line 10 was generating a never-ending series of numbers . On the
          keypress, the number currently in the variable b became the seed for
          line 30.
              Another way to provide a seed is with RANDOMIZE (note the Z);
          change the program once again to :-
              10 randomize
              20 for a=1 to 10
              30 print int(rnd*6+1);
              40 next
          When run this program asks for a seed. Put in any reasonable number
          and a series will be printed which has been seeded by your number.
              Finally change the program once again to:-

              10 randomize(peek(64504)): rem double brackets
              20 for a=1 to 10
              30 print int(rnd(a)*6+1);
              40 next

              When run, this program will usually generate a different series
          each time, because the seed has been formed by accessing the
          internal clock of the computer. The clock ticks out the seconds all
          the time the computer is running and RND takes the number from 0 to
          60 which PEEK(64504) offers it as it's seed. (PEEK accesses the
          'works' of the computer, 64504 is the address at which the clock is
          found).
              Although I have shown how to write simple code to insert into a
          program to simulate throwing a dice, there are other requirements
          for random numbers. Why not experiment using the above short
          programs to find out more about the subject. For example, in
          INT(RND*6+1) why is the +1 necessary?;try leaving it out and see
          what happens. By the way, INT simply forces the generated numbers
          into whole numbers, or integers. Try leaving it out to see what
          happens.
              Before starting to write any program of consequence it is
          necessary to make notes on  what you want to achieve and how you
          intend to set about it, and game programs are no exception. Even
          when writing a program for a wellknown game like battleships or
          noughts and crosses, you may think you know all about the game, but
          do you really ?. You know well enough what you want to achieve, but
          you will need to analyse the game step by step and consider how you
          intend to code each part, which commands are suitable, how to comply
          with each rule and still make the game interesting.

          SNAP   Let us consider the game of 'Snap' for two players.  There
          ====   could be two identical lists of names which are displayed at
          random on the screen.  When two identical names are displayed
          together the player who first shouts 'SNAP' wins that round. Simple
          enough, but let's think around the idea.
          1)  We could use names of animals, flowers or playing cards. For
              computer addicts we could even use Bytes in Binary.
          2)  How many names make a good game ?5?, 10? ; let's try 10 for
              starters. The names could be stored in two identical arrays, say
              list1$(10) and list2$(10). One set of DATA lines, READ in list1$
              then RESTORE and READ in list2$. Names can be selected randomly
          3)  What about screen layout?  Should we show one name each side of
              the screen? Should we clear the screen after each pair has been
              displayed, or wait for a 'SNAP'. Should we show the player's
              names?  What about scoring?.  Do a rough layout on paper.
          4)  What about player participation? Can we give each player two
              keys; one to display the next name, the other to indicate
              'SNAP'? How can this be achieved.
          5)  How do we score? Can we arrange for the scoring to be automatic,
              with a player's name printed as winner.

              The above are just the first few questions I would ask myself if
          I wanted to write a 'simple' game of SNAP. The answers, together
          with a minimum of programming would tell me if the game was worth
          proceeding with. I think that it could be a good game for young
          children, given an attractive screen layout. Why not try to write
          it? Can you think of variations ?  For example, a game against the
          computer with a time delay to slow up the computer and give the
          player a chance.

              This is an example of the sort of questions that have to be
          asked before starting any program. Once started you can be sure that
          there will be plenty more questions to be asked and decisions to be
          made.

          Snakes and Ladders  Can you devise a way of presenting Snakes and
          ==================  Ladders on screen without a representation of a
          board on the screen? Perhaps not, but visualise a path, say 100
          steps long littered with roller skates and banana skins. Step on a
          roller skate and you career forwards a random number of steps; step
          on a banana skin and you slip backwards a step or two. Now it can be
          done!.

          Not interested?  Then have a look in books on puzzles and games for
          inspiration. Who knows, you may come up with a best-seller.
          Remember, it's not the complexity of a game that matters, but
          whether it's addictive. Don't make a game too easy, but strike a
          balance that keeps the punter coming back for more.

          DARTS  On this disc you will find DARTS.BAS which I wrote. Not being
          =====  able to show a dart-board on screen presented a problem, but
          as long as the players know the game DARTS.BAS can be quite amusing.
         I had to ask myself the sort of questions that were posed above for
          SNAP, and decide on the ground rules.  Having watched professional
          darts on TV, I imagined a good player stepping up to throw. Suppose
          treble twenty is aimed for. Sometimes it will be scored, but most
          times the dart will miss and score single twenty, single one or five
          or treble one or five.(If you don't get this, have a look at a
          standard dartboard for the arrangement of the 'beds'). Taking say,
          twenty throws, I decided that a good player might hit the treble
          twenty bed four times, get single twenty twelve times,  and so on
          and I had to find a way to do this for each bed, including the inner
          and outer bull. In addition I decided to have two players, show a
          running total of scores and the number still required to win, plus
          the score of each dart and the last three darts for each player. The
          rules of darts were to be observed, including finishing on a double,
          and forfeiting throws if the total needed was exceeded.

          After some false starts, I found a system which seemed to have
          promises, and having written some exploratory code, I decided to go
          ahead. If you get a listing of DARTS.BAS for reference, I will
          explain points of interest. If you can also get a look at a
          dartboard to see how each sector is numbered it will also help to
          understand the program.

          The first problem was to identify each sector and to do this I made
          two numerical ARRAYS, one holding the scoring values, going
          clockwise round the board from 1, called  'ref()', and the other
          'sector()'indicating the position of each scoring value going
          clockwise round the board. The arrays are ;

            (a)      (0)  (1)  (2)  (3)  (4)  (5)  (6)  (7)  (8)  (9)  (10)
          ref(a)     20 ,  1 , 18 ,  4 , 13 ,  6 , 10 , 15 ,  2 , 17,   3
          sector(a)        1 ,  8 , 10 ,  3 , 19 ,  5 , 12 , 14 , 17 ,  6,

            (a)      (11)(12) (13) (14) (15) (16) (17) (18) (19) (20) (21)
          ref(a)      19,  7 , 16 ,  8 , 11 , 14 ,  9 , 12 ,  5 , 20 ,  1
          sector(a)   15, 18 ,  4 , 16 ,  7 , 13 ,  9 ,  2 , 11 , 20 .

          The table above represents the two arrays. The numbers in brackets
          represent the number in the array e.g  ref(7) = 15, sector(15) = 7,
          and the relationship shown in the two examples is not coincidence as
          you can check. If a scoring value is specified, say 7, then
          sector(7)=12, the position of bed 15 on the board. Add 1 to 12 and
          ref(13) = 16, the next bed clockwise. Similarly ref(12-1)= 19, one
          bed anticlockwise. The two extra numbers in the ref() array are to
          complete the circle.

          The arrays are to be found between lines 1000 and 1160. Note how
          sector() is counted 1 to 20, whilst ref() is counted 0 to 21 to
          achieve the relationships shown in the table.

          Line 1200...  needed1 and needed2 are score still needed by players
          ===========   1 and 2. Changing the value determines the length of
          game; say 101, 301 or 501 up and this is also reflected in line 2030
          Lines 2000 - 2100  Screen display and management.
          Lines 2500 - 2620  Naming players and order of play
          Lines 3000 - 3160  Scoring for first player
          Lines 4000 - 4160  Scoring for second player
          Lines 4500 - 4550  Nominating the bed to be aimed for.
          Lines 4570 - 4760  Processing the selected bed, generating a chance
                             factor, then going to subroutine for type of bed.
          Lines 5000 - 7040  Subroutines for assessing scores. Note that this
                             is where the arrays are used e.g lines
                             5530 - 5550
          Lines 7050 - END   Tidying up and invitation for another game.

          Although the program listing looks complicated, it is really fairly
          elementary coding and when tackled a bit at a time is not difficult,
          given some determination and patience. Notice that most of the
          coding is in setting out the display etc. and a relatively small
          amount does the actual work. See how I have left plenty of space
          between modules and have chosen line number spacings to enable EDIT
          and AUTO to save typing. DARTS.BAS is capable of much improvement.
          Why not have a try, then save it under another name?. Can you come
          up with different ways of achieving results ?

          Adventure Games  You can't have missed seeing these reviewed in
          ===============  magazines and have maybe played them yourself,
          involving collecting items and solving puzzles to reach a goal. I am
          convinced that somehow there is a variation on the theme that could
          start a new trend. Think about it.

          Summing up     The possibilities for writing games is endless. They
          ==========     are satisfying to write as you have an end product
          that can keep somebody amused for hours and you also get the
          opportunity for some real problem solving, decision making and loads
          of good practice.

          End of MORE.J
 opportunity for some real problem solving, decision making and lo